local addon = MissionMinder:CreateAddon("MissionMinder-Garrison")
local module = addon:CreateModule("missionPlanner")

---------------------------------------------
-- INITIALIZE
---------------------------------------------
function module.initialize()
	local missionFrame = GarrisonMissionFrame
	local missionPage = missionFrame:GetMissionPage()

--[[module:RegisterEvent("GARRISON_FOLLOWER_LIST_UPDATE", dirtyMissionCounterCache);
	module:RegisterEvent("GARRISON_FOLLOWER_XP_CHANGED", dirtyMissionCounterCache);
	module:RegisterEvent("GARRISON_FOLLOWER_REMOVED", dirtyMissionCounterCache);

	-- Mission tooltips
	local missionList = GarrisonMissionFrame.MissionTab.MissionList
	local missionButtons = missionList.listScroll.buttons;
	for i = 1, #missionButtons do
		local button = missionButtons[i]
		button:HookScript("OnEnter", createMissionTooltip);
	end
	hooksecurefunc("GarrisonMissionButton_OnEnter", createMissionTooltip)

	-- Periodically update mission tooltips
	for i = 1, #missionButtons do
		missionButtons[i].UpdateTooltip = GarrisonMissionButton_OnEnter
	end

	-- Update mission tooltip mechanics with minimized party counters
	GarrisonMission_DetermineCounterableThreats = determineCountersFromMinimizedParty
]]
	-- Load minimized mission party
	-- MissionList is initialized early, no need to hook OrderHallMissionFrame:SetupMissionList()
	local missionButtons = missionFrame.MissionTab.MissionList.listScroll.buttons or {}
	for i = 1, #missionButtons do
		local button = missionButtons[i]
		button:HookScript("OnClick", MissionMinder_LoadSavedParty);
	end

	-- Show mission expiration
	for i = 1, #missionButtons do
		local button = missionButtons[i]
		button.Expires = button:CreateFontString("MissionExpires", "ARTWORK", "GameTooltipText")
		button.Expires:SetPoint("BOTTOM", button.MissionType);
		button.Expires:Show();
	end
	hooksecurefunc(missionFrame.MissionTab.MissionList, "Update", MissionMinder_AvailableMissionList_ShowTimeRemaining)
	--missionList.listScroll.update = GarrisonMissionList_Update;  -- Set the update method to the new hooked method

	-- Show minimized parties on mission button
	hooksecurefunc(missionFrame.MissionTab.MissionList, "Update", MissionMinder_AvailableMissionList_ShowMinimizedParty);
	--missionList.listScroll.update = GarrisonMissionList_Update;  -- Set the update method to the new hooked method]]

	-- Minimize mission when right click on MissionPage
	missionPage:SetScript("OnClick", function() missionPage.MinimizeButton:Click(); end);

	-- Minimize mission on FollowerFrame RightClick
	local partyFollowers = missionPage.Followers;
	for i = 1, #partyFollowers do
		partyFollowers[i]:SetScript("OnMouseUp", MissionMinder_MissionPageFollowerFrame_OnMouseUp)
	end

	-- Follower tooltips
	--[[local followerButtons = FollowerList.listScroll.buttons;
	for i = 1, #followerButtons do
		local button = followerButtons[i]
		button:SetScript("OnEnter", createFollowersTooltip);
		button:SetScript("OnLeave", hideMissionCountersTooltip);
	end]]

	-- Double click to assign followers to mission
	missionPage:HookScript("OnHide", MissionMinder_FollowerList_DisableSingleClickTimer)
	MissionMinder_FollowerList_Initialize(missionFrame.FollowerList, function()
		local followerButtons = missionFrame.FollowerList.listScroll.buttons
		for i = 1, #followerButtons do
			local button = followerButtons[i].Follower
			button.OriginalOnClick = button:GetScript("OnClick")
			button:SetScript("OnClick", MissionMinder_FollowerList_SingleClick);
			button:SetScript("OnDoubleClick", MissionMinder_FollowerList_AssignFollowerToMission);
		end
	end)

	-- Show remaining XP to level on follower list button
	hooksecurefunc(missionFrame.FollowerList, "UpdateData", MissionMinder_FollowerList_UpdateXPRemaining)
	MissionMinder_FollowerList_Initialize(missionFrame.FollowerList, function()
		local followerButtons = missionFrame.FollowerList.listScroll.buttons
		for i = 1, #followerButtons do
			local button = followerButtons[i].Follower
			local xpRemaining = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
			xpRemaining:SetPoint("BOTTOMRIGHT", button.BG);
			xpRemaining:SetText(nil);
			xpRemaining:Show();
			button.xpRemaining = xpRemaining;
		end
	end)

	-- Color follower levels by bias
	hooksecurefunc(missionFrame.FollowerList, "UpdateData", MissionMinder_FollowerList_UpdateBiasColorFollowerLevels)

	-- Change In Party follower busy frame color
	--[[hooksecurefunc("GarrisonFollowerList_Update", updateInPartyFollowerBusyFrameColor)

	-- Drag-to-swap party members
	module.GarrisonMissionPage_SetFollower = GarrisonMissionPage_SetFollower
	GarrisonMissionPage_SetFollower = setFollower
	GarrisonFollowerPlacerFrame:HookScript("OnClick", clearDragSource);
	for i = 1, #partyFollowers do
		partyFollowers[i]:HookScript("OnDragStart", setDragSource);
		partyFollowers[i]:HookScript("OnReceiveDrag", clearDragSource);
	end
	for i = 1, #followerButtons do
		local button = followerButtons[i]
		button:HookScript("OnDragStart", clearDragSource)
	end]]

	-- Follower level up
	hooksecurefunc(missionFrame.FollowerList, "UpdateData", MissionMinder_FollowerList_UpdateFollowerLevelUp)
	hooksecurefunc(missionFrame, "AssignFollowerToMission", MissionMinder_MissionFrame_UpdatePartyLevelUp);
	hooksecurefunc(missionFrame, "RemoveFollowerFromMission", MissionMinder_MissionFrame_UpdatePartyLevelUp);
	MissionMinder_FollowerList_Initialize(missionFrame.FollowerList, function()
		local followerButtons = missionFrame.FollowerList.listScroll.buttons
		for i = 1, #followerButtons do
			local button = followerButtons[i].Follower
			local levelUp = button.PortraitFrame:CreateTexture();
			levelUp:SetPoint("BOTTOM", button.PortraitFrame, "BOTTOMLEFT")
			levelUp:Hide();
			button.levelUp = levelUp
		end
	end)
	for i = 1, #partyFollowers do
		local levelUp = partyFollowers[i].PortraitFrame:CreateTexture();
		levelUp:SetPoint("BOTTOM", partyFollowers[i].PortraitFrame, "BOTTOMLEFT")
		levelUp:Hide();
		partyFollowers[i].levelUp = levelUp
	end

	-- Estimate party member XP gains
	hooksecurefunc(missionFrame, "AssignFollowerToMission", MissionMinder_MissionPage_UpdatePartyXPGains);
	hooksecurefunc(missionFrame, "RemoveFollowerFromMission", MissionMinder_MissionPage_UpdatePartyXPGains);
	for i = 1, #partyFollowers do
		local xpGain = partyFollowers[i]:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
		xpGain:SetPoint("TOP", partyFollowers[i], "BOTTOM")
		xpGain:SetTextColor(0, 1, 0)
		xpGain:SetText(nil)
		xpGain:Show()
		partyFollowers[i].XPGain = xpGain
	end

	-- Periodically update follower list (for mission time remaining)
	--[[FollowerList:SetScript("OnUpdate", followerList_OnUpdate);]]

	-- Show reward quality borders
--	hooksecurefunc("GarrisonMissionButton_SetRewards", MissionButton_updateRewardQualityBorder);
	hooksecurefunc("GarrisonLandingPageReportList_UpdateItems", MissionMinder_LandingPageReport_AvailableMissions_UpdateRewardQualityBorder);
	GarrisonLandingPageReport.List.listScroll.update = GarrisonLandingPageReportList_UpdateItems
	--hooksecurefunc("GarrisonMissionPage_SetReward", updateRewardQualityBorder);

	-- Clear all parties when mission frame closes
	missionFrame:HookScript("OnHide", MissionMinder_MissionFrame_ClearAllParties)

	-- Change order of followers to better aid mission planning
--[[	module.GarrisonFollowerList_SortFollowers = GarrisonFollowerList_SortFollowers
	GarrisonFollowerList_SortFollowers = sortFollowersForMission

	-- Update mission buttons for party
	hooksecurefunc("GarrisonMissionList_Update", updateMissionButtonsForParty)
	missionList.listScroll.update = GarrisonMissionList_Update;  -- Set the update method to the new hooked method

	-- Update mission button rewards to include currencyQuantity
	hooksecurefunc("GarrisonMissionButton_SetRewards", setRewardCurrencyQuantity)


	]]

	missionPage:SetScript("OnKeyUp", MissionMinder_MissionPage_SpaceBarStartMission)
end
