local addon = MissionMinder:CreateAddon("MissionMinder")
local module = addon:CreateModule("listView")


---------------------------------------------
-- LOCALE
---------------------------------------------
local L = MissionMinder.locale



---------------------------------------------
-- Data
---------------------------------------------
local completingMissionID = nil

---------------------------------------------
-- List view
---------------------------------------------
function MissionMinder_MissionIsComplete(mission)
	if ( mission.completed ) then
		return true
	elseif ( mission.followers and #mission.followers > 0 ) then
		local missionTimeLeft = C_Garrison.GetFollowerMissionTimeLeftSeconds(mission.followers[1]);
		return missionTimeLeft and missionTimeLeft <= 0
	end

	return false;
end


---------------------------------------------
-- InProgress MissionList
---------------------------------------------

function MissionMinder_CompletedMissionsSortComparator(mission1, mission2)
	-- Sort In Progress missions by completed first
	if ( MissionMinder.ListViewEnabled() and mission1.inProgress ) then
		local mission1Complete = MissionMinder_MissionIsComplete(mission1);
		local mission2Complete = MissionMinder_MissionIsComplete(mission2);
		if ( mission1Complete ~= mission2Complete ) then
			return mission1Complete;
		end
	end

	-- Blizzard code below --
	if ( mission1.followerTypeID ~= mission2.followerTypeID ) then
		return mission1.followerTypeID > mission2.followerTypeID;
	end

	if ( mission1.level ~= mission2.level ) then
		return mission1.level > mission2.level;
	end

	if ( mission1.isMaxLevel ) then	-- mission 2 level is same as 1's at this point
		if ( mission1.iLevel ~= mission2.iLevel ) then
			return mission1.iLevel > mission2.iLevel;
		end
	end

	if ( mission1.durationSeconds ~= mission2.durationSeconds ) then
		return mission1.durationSeconds < mission2.durationSeconds;
	end

	if ( mission1.isRare ~= mission2.isRare ) then
		return mission1.isRare;
	end

	return strcmputf8i(mission1.name, mission2.name) < 0;
end


local function getInProgressAndCompletedMissions(missionFrame, missions)
	if ( MissionMinder.ListViewEnabled() ) then
		missionFrame.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions(missionFrame.followerTypeID);
		local completed = missionFrame.MissionComplete.completeMissions;
		for index = 1, #completed do
			local mission = completed[index];
			if ( mission.completed ) then
				mission.inProgress = true;
				mission.timeLeft = ""; -- set to non-nil to avoid nil error in blizzard code
				missions[#missions + 1] = mission;
			end
		end
	end
end


-- Pretty much the same code as GarrisonMissionListMixin:OnUpdate()
local function MissionMinder_MissionList_OnUpdate_IncludeCompletedMissions(self)
	if (self.showInProgress) then
		C_Garrison.GetInProgressMissions(self.inProgressMissions, self:GetMissionFrame().followerTypeID);

		-- Specifically add missions that are complete,but haven't been looted yet
		local missionFrame = self:GetParent():GetParent()
		getInProgressAndCompletedMissions(missionFrame, self.inProgressMissions)

		self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
		self:Update();
	else
		local timeNow = GetTime();
		for i = 1, #self.availableMissions do
			if ( self.availableMissions[i].offerEndTime and self.availableMissions[i].offerEndTime <= timeNow ) then
				self:UpdateMissions();
				break;
			end
		end
	end
	self:UpdateCombatAllyMission();
end


-- Pretty much the same code as GarrisonMissionListMixin:UpdateMissions()
function MissionMinder_InProgressMissionList_IncludeCompletedMissions(self)
	C_Garrison.GetInProgressMissions(self.inProgressMissions, self:GetMissionFrame().followerTypeID);
	C_Garrison.GetAvailableMissions(self.availableMissions, self:GetMissionFrame().followerTypeID);

	-- Specifically add missions that are complete,but haven't been looted yet
	local missionFrame = self:GetParent():GetParent()
	getInProgressAndCompletedMissions(missionFrame, self.inProgressMissions)
	Garrison_SortMissions(self.inProgressMissions);

	self:UpdateCombatAllyMission();
	Garrison_SortMissions(self.availableMissions);
	self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
	self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
	if ( #self.inProgressMissions > 0 ) then
		self.Tab2.Left:SetDesaturated(false);
		self.Tab2.Right:SetDesaturated(false);
		self.Tab2.Middle:SetDesaturated(false);
		self.Tab2.Text:SetTextColor(1, 1, 1);
		self.Tab2:SetEnabled(true);
	else
		self.Tab2.Left:SetDesaturated(true);
		self.Tab2.Right:SetDesaturated(true);
		self.Tab2.Middle:SetDesaturated(true);
		self.Tab2.Text:SetTextColor(0.5, 0.5, 0.5);
		self.Tab2:SetEnabled(false);
	end
	self:Update();
end

function MissionMinder_UpdatePartyMembers(button, followers, followerCount)
	if ( followers and not followerCount) then
		followerCount = #followers
	end

	if(followers and followerCount > 0) then
		button.Title:ClearAllPoints();
		button.Title:SetPoint("TOPLEFT", 165, -5);

		for i=1, followerCount do
			local frame = button.Party[i]
			if ( not frame ) then
				frame = CreateFrame("Frame", nil, button, "MissionListPartyMemberPortrait")
				frame:SetPoint("LEFT", button.Party[i-1], "RIGHT", 10, 0);
				tinsert(button.Party, frame)
			end

			frame:SetPartyMember(followers[i])
			frame:Show()
		end
	else
		followerCount = 0
		button.Title:ClearAllPoints();
		button.Title:SetPoint("LEFT", 165, 0);
	end

	for i=followerCount+1,#button.Party do
		local frame = button.Party[i]
		frame:SetPartyMember(nil)
		frame:Hide()
	end
end

function MissionMinder_InProgressMissionList_HighlightCompletedMissions(self, ...)
	-- Hide overlay on completed missions and highlight them as finished
	local buttons = self.listScroll.buttons;

	if ( not self.showInProgress ) then
		return
	end

	for _,button in pairs(buttons) do
		local mission = button.info
		if ( mission ) then
			local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(mission.missionID); -- Success chance may be nil if an update occurs while the mission is removed
			if ( successChance ) then
				button.SuccessChance:SetFormattedText(PERCENTAGE_STRING .. " " .. GARRISON_MISSION_CHANCE, successChance);
				button.SuccessChance:Show();
			else
				button.SuccessChance:SetText(nil);
				button.SuccessChance:Hide();
			end

			MissionMinder_UpdatePartyMembers(button, mission.followers)

			if( MissionMinder.ListViewEnabled() and MissionMinder_MissionIsComplete(mission)) then
				button.Overlay:Hide();

				button.LocBG:SetVertexColor(0.3921, 1, 1, 1); -- green blue
				button.Summary:SetText("(" .. COMPLETE .. ")");
				button.Summary:SetTextColor(0, 1, 0);
			else
				button.Summary:SetTextColor(1, 1, 1);
				button.LocBG:SetVertexColor(1, 1, 1, 1);
			end

			-- Summary should always show to the right of title
			button.Summary:ClearAllPoints();
			button.Summary:SetPoint("BOTTOMLEFT", button.Title, "BOTTOMRIGHT", 8, 0);

			-- Truncate title if too long
			--[[button.Title:SetWidth(0)
			local maxButtonWidth = 655 - (mission.numRewards * 65) - button.Summary:GetWidth() - 8
			if ( button.Title:GetWidth() > maxButtonWidth ) then
				button.Title:SetWidth(maxButtonWidth)
			end]]
		end
	end
end

local function findCompleteMission(missionFrame, missionID)
	local missionComplete = missionFrame.MissionComplete
	missionComplete.completeMissions = C_Garrison.GetCompleteMissions(missionFrame.followerTypeID);

	local missions = missionComplete.completeMissions or {}
	for i = 1, #missions do
		if ( missionID == missions[i].missionID ) then
			module:DEBUGF("findCompleteMissionIndex(%d) => %s, %d", missionID, tostring(missions[i]), i)
			return missions[i], i
		end
	end

	module:DEBUGF("findCompleteMissionIndex(%d) => nil", missionID)
	return nil
end

function MissionMinder_MissionFrame_ViewSpecificCompletedMission(self)
	if ( not MissionMinder.ListViewEnabled() ) then
		self.OriginalOnClickViewCompletedMissionsButton(self)
	elseif ( completingMissionID ) then
		if ( not MissionCompletePreload_IsReady() ) then
			self:GetCompleteDialog():Show()
			self:GetCompleteDialog().BorderFrame.ViewButton:SetEnabled(false);
			self:GetCompleteDialog().BorderFrame.LoadingFrame:Show();
			MissionCompletePreload_StartTimeout(GARRISON_MODEL_PRELOAD_TIME, MissionMinder_MissionFrame_ViewSpecificCompletedMission, self);
			return;
		end
		PlaySound("UI_Garrison_CommandTable_ViewMissionReport");

		self:GetCompleteDialog():Hide();
		self.FollowerTab:Hide();
		self.FollowerList:Hide();
		HelpPlate_Hide();
		self.MissionComplete:Show();
		self.MissionCompleteBackground:Show();

		self.MissionComplete.currentIndex = select(2, findCompleteMission(self, completingMissionID))

		self:MissionCompleteInitialize(self.MissionComplete.completeMissions, self.MissionComplete.currentIndex);
	end
end

-- When user clicks on a mission from the mission list (either available or in progress)
function MissionMinder_InProgressMissionList_ViewMission(self, button, ...)
	if ( not MissionMinder.ListViewEnabled() ) then
		return;
	end

	-- Ignore CHATLINK attempts and non-inProgress missions (handled properly already)
	if ( IsModifiedClick("CHATLINK") or not self.info.inProgress ) then
		return;
	end

	local missionFrame = self:GetParent():GetParent():GetParent():GetParent():GetParent()
	local mission = self.info;
	if ( mission and MissionMinder_MissionIsComplete(mission) ) then -- Mission must be completed
		if ( MissionMinder.QuickLoot() ) then
			MissionMinder_LootMission(mission);
		else
			completingMissionID = mission.missionID

			MissionCompletePreload_Cancel(missionFrame);
			MissionCompletePreload_LoadMission(missionFrame, completingMissionID,
				GarrisonFollowerOptions[missionFrame.followerTypeID].showSingleMissionCompleteFollower,
				GarrisonFollowerOptions[missionFrame.followerTypeID].showSingleMissionCompleteAnimation);

			-- This would be part of the OnClick event of the In Progress tab
			--[[missionFrame.FollowerTab:Hide();
			missionFrame.FollowerList:Hide();
			HelpPlate_Hide();

			missionFrame.MissionComplete:Show();
			missionFrame.MissionCompleteBackground:Show();

			local self = missionFrame.MissionComplete;
			self.currentIndex = select(2, findCompleteMission(missionFrame, mission.missionID))

			if ( self.currentIndex ) then]]
			missionFrame:OnClickViewCompletedMissionsButton(completingMissionID)
				--[[MissionCompletePreload_Cancel(missionFrame); -- Cancel any previous preload that is still running

				MissionCompletePreload_LoadMission(missionFrame, mission.missionID,
		GarrisonFollowerOptions[missionFrame.followerTypeID].showSingleMissionCompleteFollower, GarrisonFollowerOptions[missionFrame.followerTypeID].showSingleMissionCompleteAnimation);

				--MissionCompletePreload_LoadMission(mission.missionID); -- Start preload
				--GarrisonMissionComplete_Initialize(self.completeMissions, self.currentIndex);
			end]]
		end
	end
end


---------------------------------------------
-- CompleteDialog
---------------------------------------------
function MissionMinder_CompleteDialog_ShowInProgressMissionList(self, ...)
	local missionFrame = self:GetParent():GetParent():GetParent():GetParent():GetParent()
	local missionList = missionFrame.MissionTab.MissionList

	if ( MissionMinder.ListViewEnabled() ) then
		PlaySound("UI_Garrison_CommandTable_ViewMissionReport");
		GarrisonMissionListTab_SetTab(missionList.Tab2);
		missionList.CompleteDialog:Hide();
	else
		self.OriginalOnClick(self, ...);
	end
end

---------------------------------------------
-- MissionComplete
---------------------------------------------
function MissionMinder_MissionComplete_NextButton_CloseMissionComplete(self, ...)
	completingMissionID = nil

	local missionFrame = self:GetParent():GetParent()
	if ( MissionMinder.ListViewEnabled() ) then
		missionFrame:CloseMissionComplete()
	else
		self.OriginalOnClick(self, ...)
	end
end



---------------------------------------------
-- MissionListPartyMemberPortrait MixIn
---------------------------------------------
MissionListPartyMemberPortraitMixIn = {}
function MissionListPartyMemberPortraitMixIn:SetPortraitIcon(iconFileID)
	if (iconFileID == nil or iconFileID == 0) then
		-- unknown icon file ID; use the default silhouette portrait
		self.Portrait:SetTexture("Interface\\Garrison\\Portraits\\FollowerPortrait_NoPortrait");
	else
		self.Portrait:SetTexture(iconFileID);
	end
end

function MissionListPartyMemberPortraitMixIn:SetQuality(quality)
	local color = quality and ITEM_QUALITY_COLORS[quality] or nil;
	if (color) then
		self:SetQualityColor(color.r, color.g, color.b);
	else
		self:SetQualityColor(1, 1, 1);
	end
end

function MissionListPartyMemberPortraitMixIn:SetQualityColor(r, g, b)
	self.PortraitRingQuality:SetVertexColor(r, g, b);
end

function MissionListPartyMemberPortraitMixIn:SetPartyMember(followerID)
	if ( followerID ) then
		local info = C_Garrison.GetFollowerInfo(followerID)
		self.info = info
		self:SetQuality(info.quality);
		self:SetPortraitIcon(info.portraitIconID);
		self.info = nil;
	else
		self:SetQualityColor(0.3, 0.3, 0.3);
		self:SetPortraitIcon(nil);
	end
end

function module.register(missionFrame)
	local missionList = missionFrame.MissionTab.MissionList

	-- "View Completed Missions" button will open "In Progress" tab instead of "CompleteMission" frame
	local viewButton = missionList.CompleteDialog.BorderFrame.ViewButton;
	viewButton.OriginalOnClick = viewButton:GetScript("OnClick")
	viewButton:SetScript("OnClick", MissionMinder_CompleteDialog_ShowInProgressMissionList);

	-- Override CompleteDialog initialize so we can target a specific mission
	--instead of defaulting to "first mission in list"
	missionFrame.OriginalOnClickViewCompletedMissionsButton = missionFrame.OnClickViewCompletedMissionsButton
	missionFrame.OnClickViewCompletedMissionsButton = MissionMinder_MissionFrame_ViewSpecificCompletedMission

	-- Completed Missions that haven't been looted are not considered "in progress" anymore
	--and need to be added to the list
	missionList.UpdateMissions = MissionMinder_InProgressMissionList_IncludeCompletedMissions
	missionList.OnUpdate = missionListOnUpdate_IncludeCompletedMissions
	missionList:SetScript("OnUpdate", missionList.OnUpdate)

	-- Sort InProgress missions by completed first
	Garrison_SortMissions = function(missions)
		table.sort(missions, MissionMinder_CompletedMissionsSortComparator);
	end

	-- Highlight completed missions
	hooksecurefunc(missionList, "Update", MissionMinder_InProgressMissionList_HighlightCompletedMissions);
	--GarrisonMissionFrame.MissionTab.MissionList.listScroll.update = GarrisonMissionList_Update; -- Set the update method to the new hooked method

	-- Clicking a completed mission will view it
	local missionButtons = missionList.listScroll.buttons;
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button:HookScript("OnClick", MissionMinder_InProgressMissionList_ViewMission);
	end

	-- Show special tooltips for completed missions
	--[[GarrisonMissionButton_SetInProgressTooltip = inprogressTooltips]]

	-- Create SuccessChance text
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		local successChance = button:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
		successChance:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -17);
		successChance:SetTextColor(0, 1, 0);
		successChance:SetSize(115, 15)
		button.SuccessChance = successChance;
	end

	-- Create Party member frames
	for i = 1, #missionButtons do
		local button = missionButtons[i];
		button.Party = {}
		local partyMember = CreateFrame("Frame", nil, button, "MissionListPartyMemberPortrait")
		partyMember:SetPoint("LEFT", button.SuccessChance, "RIGHT", 20, 0);
		tinsert(button.Party, partyMember)
	end

	-- rewrite "Next" mission button as "Close"
	module:WatchOption("ListViewEnabled", function(option, value)
		local nextMissionButton = missionFrame.MissionComplete.NextMissionButton;
		nextMissionButton:SetText( value and CLOSE or NEXT )
	end)

	-- Clicking "Next" misison button closes the mission complete frame
	local nextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton
	nextMissionButton.OriginalOnClick = nextMissionButton:GetScript("OnClick")
	nextMissionButton:SetScript("OnClick", MissionMinder_MissionComplete_NextButton_CloseMissionComplete);

	-- Refresh completed missions list on GarrisonMissionFrame.MissionComplete:Hide
	--[[local events = MissionMinder.events;
	events:RegisterEvent("listview", "GARRISON_MISSION_COMPLETE_RESPONSE", refreshCompletedMissionsForQuickLoot);
	events:RegisterEvent("listview", "GARRISON_MISSION_BONUS_ROLL_COMPLETE", refreshCompletedMissionsForQuickLoot);
	GarrisonMissionFrame.MissionComplete:HookScript("OnHide", refreshCompletedMissions);

	-- Closing GarrisonMissionFrame should select the "Available Missions" tab. This will cause the next open to show
	--the completed missions dialog again (if applicable)
	GarrisonMissionFrame:HookScript("OnHide", selectAvailableMissions);

	-- Mission finished event should not show complete dialog if "In Progress" tab already selected
	-- Currently uses a pre-hook, with more work might be able to remove the pre-hook requirement and use secure post-hook
	addon.GarrisonMissionFrame_CheckCompleteMissions = GarrisonMissionFrame_CheckCompleteMissions;
	GarrisonMissionFrame_CheckCompleteMissions = showMissionCompleteFrame;]]
end
