local addon = MissionMinder:CreateAddon("MissionMinder")
local module = addon:CreateModule("options")


---------------------------------------------
-- Locale
---------------------------------------------
local L = MissionMinder.locale;
local L_CONFIGURE_MISSIONMINDER = L["Configure MissionMinder"]
local L_ADDON_DESCRIPTION = L["Manage garrison missions in a more streamline, less-invasive manner"]
local L_LISTVIEW = L["List View"]
local L_LISTVIEW_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_ENABLED = L["Enabled"]
local L_LISTVIEW_ENABLED_DESCRIPTION = L["View completed missions as a list from the \"In Progress\" tab"]
local L_QUICKLOOT = L["Quick loot"]
local L_QUICKLOOT_DESCRIPTION = L["Disable the completion frame entirely, skip all the fanfare, and just give me the loot already!"]
local L_SHOWXP = L["Show follower XP gains"]
local L_SHOWXP_DESCRIPTION = L["Show messages regarding individual follower XP gains"]
local L_GENERAL_OPTIONS = L["General Options"]
local L_SKIPFIGHTANIMATIONS = L["Skip fight animations"]
local L_SKIPFIGHTANIMATIONS_DESCRIPTION = L["Skip all fight animations and go straight to rewards"]
local L_AUTOOPENREWARDS = L["Automatically open rewards chest"]
local L_AUTOOPENREWARDS_DESCRIPTION = L["Remove the need to click on the rewards chest to recieve the rewards"]
local L_AUTOCLOSE = L["Automatically close on loot"]
local L_AUTOCLOSE_DESCRIPTION = L["Automatically close the completion dialog if all actions have been completed for a mission"]
local L_PLANNER = L["Planner"];
local L_SHOWFOLLOWERCOUNTERTOTALS = L["Show active follower counter totals"];
local L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION = L["Easily track how many of each counter you currently have active"]
local L_IGNOREXPGAINTRAITS = L["Ignore XP increase traits"]
local L_IGNOREXPGAINTRAITS_DESCRIPTION = L["Do not show XP increase traits on missions that do not reward Bonus XP"]


---------------------------------------------
-- OPTIONS
---------------------------------------------
local VERSION = "Version"
local DEBUG = "debug";
local LISTVIEW_ENABLED = "ListViewEnabled"
local AUTO_OPEN_REWARDS = "AutoOpenRewards";
local AUTO_CLOSE = "AutoClose";
local SKIP_FIGHT_ANIMATIONS = "SkipFightAnimations";
local QUICK_LOOT = "QuickLoot";
local SHOW_FOLLOWER_XP_GAINS = "ShowFollowerXPGains";
local SHOW_FOLLOWER_COUNTER_TOTALS = "ShowFollowerCounterTotals";
local IGNORE_XPGAINS_TRAITS = "IgnoreXPGainTraits";

local INPROGRESS_MISSIONS = "InProgressMissions";

MissionMinder:SetDefaultOptions({
	[VERSION] = 1, -- Options Versioning
	[DEBUG] = true,
	[LISTVIEW_ENABLED] = true,
	[AUTO_OPEN_REWARDS] = false,
	[AUTO_CLOSE] = false,
	[SKIP_FIGHT_ANIMATIONS] = false,
	[QUICK_LOOT] = false,
	[SHOW_FOLLOWER_XP_GAINS] = false,
	[SHOW_FOLLOWER_COUNTER_TOTALS] = true,
	[IGNORE_XPGAINS_TRAITS] = true
})

local function getOption(info)
	return MissionMinder:GetOption(info.arg)
end

local function setOption(info, value)
	return MissionMinder:SetOption(info.arg, value)
end

function MissionMinder.ListViewEnabled()
	return MissionMinder:GetOption(LISTVIEW_ENABLED);
end

function MissionMinder.AutoLootRewards()
	return MissionMinder:GetOption(AUTO_OPEN_REWARDS);
end

function MissionMinder.AutoClose()
	return MissionMinder:GetOption(AUTO_CLOSE);
end

function MissionMinder.SkipFightAnimations()
	return MissionMinder:GetOption(SKIP_FIGHT_ANIMATIONS);
end

function MissionMinder.QuickLoot()
	return MissionMinder.ListViewEnabled() and MissionMinder:GetOption(QUICK_LOOT);
end

function MissionMinder.ShowFollowerXPGains()
	return MissionMinder.QuickLoot() and MissionMinder:GetOption(SHOW_FOLLOWER_XP_GAINS);
end

function MissionMinder.ShowFollowerCounterTotals()
	return MissionMinder:GetOption(SHOW_FOLLOWER_COUNTER_TOTALS);
end

function MissionMinder.IgnoreXPGainTraits()
	return MissionMinder:GetOption(IGNORE_XPGAINS_TRAITS);
end

---------------------------------------------
-- OPTIONS PANE
---------------------------------------------
local configName = string.format("%sOptionsPane", addon.name);
LibStub("AceConfig-3.0"):RegisterOptionsTable(configName, {
	name = addon.name  .. " " .. addon.version,
	desc = L_ADDON_DESCRIPTION,
	type = "group",
	args = {
		desc = {
			order = 1,
			type = "description",
			name = L_ADDON_DESCRIPTION,
			width = "full"
		};
		emptySpace = {
			order = 3,
			type = "description",
			name = " ",
			width = "full"
		};
		listview = {
			order = 10,
			type = "group",
			inline = true,
			name = L_LISTVIEW,
			args = {
				description = {
					order = 1,
					type = "description",
					name = L_LISTVIEW_DESCRIPTION,
					width = "full",
				};
				spacer = {
					order = 2,
					type = "description",
					name = " ",
					width = "full",
				};
				listviewEnabled = {
					order = 10,
					name = L_ENABLED,
					desc = L_LISTVIEW_ENABLED_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = LISTVIEW_ENABLED
				};
				quickLoot = {
					order = 25,
					name = L_QUICKLOOT,
					desc = L_QUICKLOOT_DESCRIPTION,
					disabled = function() return not MissionMinder.ListViewEnabled(); end,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = QUICK_LOOT
				};
				showXP = {
					order = 30,
					name = L_SHOWXP,
					desc = L_SHOWXP_DESCRIPTION,
					disabled = function() return not MissionMinder.ListViewEnabled() or not MissionMinder.QuickLoot(); end,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = SHOW_FOLLOWER_XP_GAINS
				};
			}
		};
		general = {
			order = 15,
			type = "group",
			inline = true,
			disabled = MissionMinder.QuickLoot,
			name = L_GENERAL_OPTIONS,
			args = {
				skipFightAnimations = {
					order = 5,
					name = L_SKIPFIGHTANIMATIONS,
					desc = L_SKIPFIGHTANIMATIONS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = SKIP_FIGHT_ANIMATIONS
				};
				autoOpenRewards = {
					order = 10,
					name = L_AUTOOPENREWARDS,
					desc = L_AUTOOPENREWARDS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = AUTO_OPEN_REWARDS
				};
				autoClose = {
					order = 15,
					name = L_AUTOCLOSE,
					desc = L_AUTOCLOSE_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = AUTO_CLOSE
				};
			}
		};
		planner = {
			order = 20,
			type = "group",
			inline = true,
			name = L_PLANNER,
			args = {
				showFollowerCounterTotals = {
					order = 5,
					name = L_SHOWFOLLOWERCOUNTERTOTALS,
					desc = L_SHOWFOLLOWERCOUNTERTOTALS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = function(info, value)
						setOption(info, value);
						if ( GarrisonMissionFrame:IsShown() and GarrisonMissionFrame.FollowerTab:IsShown()) then
							if(value and not MissionMinderActiveFollowerCounters:IsShown() ) then
								MissionMinderActiveFollowerCounters:Show();
								if ( MissionMinder.ptr ) then
									GarrisonThreatCountersFrame:Hide();
								end
							elseif(not value and MissionMinderActiveFollowerCounters:IsShown()) then
								MissionMinderActiveFollowerCounters:Hide();
								if ( MissionMinder.ptr ) then
									GarrisonThreatCountersFrame:Show();
								end
							end
						end
					end,
					get = getOption,
					arg = SHOW_FOLLOWER_COUNTER_TOTALS
				};
				ignoreXPGainTraits = {
					order = 10,
					name = L_IGNOREXPGAINTRAITS,
					desc = L_IGNOREXPGAINTRAITS_DESCRIPTION,
					type = "toggle",
					width = "full",
					set = setOption,
					get = getOption,
					arg = IGNORE_XPGAINS_TRAITS
				};
			}
		};
		--[===[@debug@
		spacer = {
			order = 9000,
			type = "description",
			name = " ",
			width = "double"
		};
		debugSpacer = {
			order = 9001,
			type = "description",
			name = " ",
			width = "normal"
		};
		debug = {
			order = 9002,
			name = DEBUG_BUILD,
			type = "toggle",
			set = setOption,
			get = getOption,
			width = "half",
			arg = DEBUG
		};
		--@end-debug@]===]
	}
});
local configPane = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(configName, addon.name);


local function showConfigPane()
	InterfaceOptionsFrame_OpenToCategory(configPane);

	-- #31: Blizzard options interface does not open pane on initial load
	if ( not configPane:IsShown() ) then
		InterfaceOptionsFrame_OpenToCategory(configPane);
	end
end


---------------------------------------------
-- MissionMinderConfigureButton MixIn
---------------------------------------------
MissionMinderConfigureButtonMixIn  = {}
function MissionMinderConfigureButtonMixIn:OnEnter()
	self.texture:SetAlpha(1.0)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(L_CONFIGURE_MISSIONMINDER)
	GameTooltip:Show()
end

function MissionMinderConfigureButtonMixIn:OnLeave()
	self.texture:SetAlpha(0.75)
	GameTooltip_Hide()
end

function MissionMinderConfigureButtonMixIn:OnMouseDown()
	self.texture:SetPoint("TOPLEFT", 1, -1)
end

function MissionMinderConfigureButtonMixIn:OnMouseUp()
	self.texture:SetPoint("TOPLEFT", 0, 0)
end

function MissionMinderConfigureButtonMixIn:OnClick()
	HideParentPanel(self)
	showConfigPane()
end

---------------------------------------------
-- INITIALIZE
---------------------------------------------
function module.initialize()
	-- Create LandingPage.ConfigureButton
	CreateFrame("BUTTON", nil, GarrisonLandingPage, "MissionMinderConfigureButtonTemplate")

	-- Enable / Disable debugging based on option
	module:WatchOption(DEBUG, function(option, value)
		MissionMinder.debug = value
	end)

	--[===[@alpha@
	SLASH_MISSIONMINDER1 = "/missionminder";
	SlashCmdList["MISSIONMINDER"] = showConfigPane
	--@end-alpha@]===]
end

function module.register(missionFrame)
	-- Create MissionFrame.ConfigureButton
	CreateFrame("BUTTON", nil, missionFrame, "MissionMinderConfigureButtonTemplate")
end
